Package: locuszoomr
Title: Gene Locus Plot with Gene Annotations
Version: 0.3.8
Authors@R: 
    c(person(given = "Myles",family = "Lewis",
    role = c("aut", "cre"),
    email = "myles.lewis@qmul.ac.uk",
    comment = c(ORCID = "0000-0001-9365-5345")))
BugReports: https://github.com/myles-lewis/locuszoomr/issues
URL: https://github.com/myles-lewis/locuszoomr
Description: Publication-ready regional gene locus plots similar to those produced by the web interface 'LocusZoom' <https://my.locuszoom.org>, but running locally in R. Genetic or genomic data with gene annotation tracks are plotted via R base graphics, 'ggplot2' or 'plotly', allowing flexibility and easy customisation including laying out multiple locus plots on the same page. It uses the 'LDlink' API <https://ldlink.nih.gov/?tab=apiaccess> to query linkage disequilibrium data from the 1000 Genomes Project and can overlay this on plots <doi:10.1093/bioadv/vbaf006>.
Language: en-gb
License: GPL (>= 3)
Encoding: UTF-8
Depends: R (>= 3.5)
biocViews:
Imports: AnnotationFilter, BiocGenerics, cowplot, dplyr, ensembldb,
        GenomeInfoDb, GenomicRanges, gggrid, ggplot2, ggrepel,
        graphics, grDevices, grid, IRanges, LDlinkR, memoise, plotly,
        rlang, rtracklayer, zoo
RoxygenNote: 7.3.2
Suggests: AnnotationHub, EnsDb.Hsapiens.v75, knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-03-03 14:04:10 UTC; myles
Author: Myles Lewis [aut, cre] (<https://orcid.org/0000-0001-9365-5345>)
Maintainer: Myles Lewis <myles.lewis@qmul.ac.uk>
Repository: CRAN
Date/Publication: 2025-03-03 14:30:02 UTC
Built: R 4.6.0; ; 2026-04-30 13:13:20 UTC; unix
