# Generated by roxygen2: do not edit by hand

S3method(summary,locus)
export(eqtl_plot)
export(genetrack_ly)
export(genetracks)
export(genetracks_grob)
export(gg_addgenes)
export(gg_genetracks)
export(gg_scatter)
export(line_plot)
export(link_LD)
export(link_eqtl)
export(link_recomb)
export(locus)
export(locus_ggplot)
export(locus_plot)
export(locus_plotly)
export(multi_layout)
export(overlay_plot)
export(quick_peak)
export(scatter_plot)
export(scatter_plotly)
export(set_layers)
importFrom(AnnotationFilter,AnnotationFilterList)
importFrom(AnnotationFilter,ExonEndFilter)
importFrom(AnnotationFilter,ExonStartFilter)
importFrom(AnnotationFilter,GeneIdFilter)
importFrom(AnnotationFilter,GeneNameFilter)
importFrom(AnnotationFilter,SeqNameFilter)
importFrom(AnnotationFilter,TxEndFilter)
importFrom(AnnotationFilter,TxStartFilter)
importFrom(BiocGenerics,end)
importFrom(BiocGenerics,start)
importFrom(GenomeInfoDb,"genome<-")
importFrom(GenomeInfoDb,genome)
importFrom(GenomeInfoDb,seqlengths)
importFrom(GenomeInfoDb,seqnames)
importFrom(GenomicRanges,GRanges)
importFrom(IRanges,IRanges)
importFrom(LDlinkR,LDexpress)
importFrom(LDlinkR,LDmatrix)
importFrom(LDlinkR,LDproxy)
importFrom(cowplot,plot_grid)
importFrom(dplyr,bind_rows)
importFrom(ensembldb,ensemblVersion)
importFrom(ensembldb,exons)
importFrom(ensembldb,genes)
importFrom(ensembldb,organism)
importFrom(gggrid,grid_panel)
importFrom(ggplot2,aes)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer_scales)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_shape_manual)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,sec_axis)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,unit)
importFrom(ggplot2,xlab)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylim)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,adjustcolor)
importFrom(grDevices,col2rgb)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,rainbow)
importFrom(grDevices,rgb)
importFrom(graphics,axTicks)
importFrom(graphics,axis)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot.new)
importFrom(graphics,points)
importFrom(graphics,rect)
importFrom(graphics,strwidth)
importFrom(graphics,text)
importFrom(grid,gList)
importFrom(grid,gTree)
importFrom(grid,gpar)
importFrom(grid,polylineGrob)
importFrom(grid,rectGrob)
importFrom(grid,textGrob)
importFrom(grid,viewport)
importFrom(grid,xaxisGrob)
importFrom(memoise,drop_cache)
importFrom(memoise,memoise)
importFrom(plotly,"%>%")
importFrom(plotly,add_segments)
importFrom(plotly,add_text)
importFrom(plotly,add_trace)
importFrom(plotly,plot_ly)
importFrom(plotly,plotly_build)
importFrom(plotly,plotly_empty)
importFrom(rlang,.data)
importFrom(rtracklayer,browserSession)
importFrom(rtracklayer,getTable)
importFrom(rtracklayer,ucscTableQuery)
importFrom(stats,relevel)
importFrom(zoo,na.approx)
