#' SLE GWAS data subset
#'
#' Dataset of SNPs at 3 gene loci (UBE2L3, STAT4, IRF5) from GWAS on SLE
#' (Bentham et al, 2015, Nature Genetics 47(12):1457-64, PMID: 26502338).
#'
#' @format Data frame with 1990 rows and 11 variables
#' @source \url{https://www.ebi.ac.uk/gwas/studies/GCST003156}
#' @usage data(SLE_gwas_sub)
"SLE_gwas_sub"
